/*
 * RS485_fifo.c
 *
 *  Created on: 29 pa 2015
 *      Author: SK
 */

#include "SystemInit.h"
#include "RS485_fifo.h"

volatile TBUF Buffer;

/*******************************************************************************
* Function Name  : USART1_IRQHandler
* Description    : This function handles USART1 global interrupt request.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART1_IRQHandler(void)
{
	GPIO_WriteBit( LED_GPIO, ERR_LED, 0 );  //DEBUG

	if( USART_GetITStatus(USART1, USART_IT_TXE) == SET )
	{
		USART_ClearITPendingBit(USART1, USART_IT_TXE);
		// sprawdzamy czy indeksy s rne
		if( Buffer.TxBeg != Buffer.TxEnd )
		{
			Buffer.TxEnd = (Buffer.TxEnd + 1) & TX_BUF_MASK;		// obliczamy i zapamitujemy nowy indeks konca (moe si zrwna z poczatkiem)
			USART1->DR = Buffer.TxBuf[Buffer.TxEnd];				// zwracamy bajt pobrany z bufora  jako rezultat funkcji
		}
		else
		{
			USART_ITConfig(USART1, USART_IT_TXE, DISABLE);  	// wylacz przerwanie = koniec transmisji
		}
	}

	if( USART_GetITStatus(USART1, USART_IT_TC) == SET )	//gdy bufor nadawczy pusty
	{
		SendMode(FALSE);	// odbiornik zaaczony && nadajnik wyaczony
	}

	if( USART_GetITStatus(USART1, USART_IT_RXNE) == SET )
	{
		register uint8_t tmp_head;
	    register uint8_t data;

	    tmp_head = (Buffer.RxBeg + 1) & RX_BUF_MASK;				// obliczamy nowy indeks poczatku
   	    data = (uint8_t)(USART1->DR);

	    if( tmp_head == Buffer.RxEnd )	Buffer.RxBeg = Buffer.RxEnd;	// sprawdzamy nadmisanie

	    else
	    {
	    	switch( data )
	    	{
	        	case 0:	break;		// ignorujemy bajt = 0
	        	case 10: break;			// ignorujemy znak LF
	        	case 13: Buffer.ascii_line++;	// sygnalizujemy obecno kolejnej linii w buforze
	        	default : Buffer.RxBeg = tmp_head; Buffer.RxBuf[tmp_head] = data;
	        }
        }
	}
}

/*******************************************************************************
* Function Name  : USART_SendChar
* Description    : This function
* Input          : Name of USART interface , Char for send
* Output         : None
* Return         : None
*******************************************************************************/
void USART_SendChar(USART_TypeDef* USARTx, uint8_t Char)
{
	/* Check the parameters */
	assert_param(IS_USART_ALL_PERIPH(USARTx));
	assert_param(IS_USART_DATA(Char));

	/* Fill Tx buffor */
	uint8_t tmp_head;
	tmp_head = (Buffer.TxBeg+1) & TX_BUF_MASK;
	while(tmp_head == Buffer.TxEnd) {}
	Buffer.TxBuf[tmp_head]=Char;
	Buffer.TxBeg=tmp_head;

	/* Turn On Tx Interrupt / Starting transmit */
	USART_ITConfig(USARTx, USART_IT_TXE, ENABLE);
}

void USART_ftoa(USART_TypeDef* USARTx, float f, char* buffer)
{
	uint32_t tmp=f*10000;
	uint8_t cz_d, cz_u;

	cz_d = f;
	cz_u = (tmp/100) % 10;

	USART_Puts( USARTx, itoa(cz_d, buffer, 10) );
	USART_SendChar(USARTx, '.');
	USART_Puts(USARTx, itoa(cz_u, buffer, 10));
}

/*******************************************************************************
* Function Name  : UART_Puts
* Description    : This function
* Input          : Name of USART interface , Data string for send
* Output         : None
* Return         : None
*******************************************************************************/
void USART_Puts( USART_TypeDef* USARTx, char *s )	// wysya acuch z pamici RAM na UART
{
	register uint8_t c;
	while ((c = *s++)) USART_SendChar( USARTx, c );		// dopki nie napotkasz 0 wysyaj znak
}

/*******************************************************************************
* Function Name  : UART_ReciveChar
* Description    : This function
* Input          : Name of USART interface
* Output         : None
* Return         : Single char from RX buffer
*******************************************************************************/
uint8_t USART_ReceiveChar( USART_TypeDef* USARTx )
{
	/* Check the parameters */
	assert_param(IS_USART_ALL_PERIPH(USARTx));

	/* Fill Rx buffor */
	if ( Buffer.RxBeg == Buffer.RxEnd ) return -1;	// sprawdzamy czy indeksy s rwne
	Buffer.RxEnd = (Buffer.RxEnd + 1) & RX_BUF_MASK;	// obliczamy i zapamitujemy nowy indeks konca bufora (moe si zrwna z pocztkiem)

    return Buffer.RxBuf[Buffer.RxEnd];					// zwracamy bajt pobrany z bufora  jako rezultat funkcji
}

/*******************************************************************************
* Function Name  : SendMode
* Description    :
* Input          : bSend  true - set RS485 in send mode, else in rece mode
*                  -
* Output         : None
* Return         : None
*******************************************************************************/
void SendMode(bool bSend)
{
	if(bSend)
	{
		GPIO_SetBits(GPIO_485, DO_485);
	}
	else
	{
		GPIO_ResetBits(GPIO_485, DO_485);
	}
}

/*******************************************************************************
* Function Name  : RS485_Configuration
* Description    : Init uart1 using PAx, PAx pins
*                  9600, 8, 1, none flow control
*                  DE/nRE using PAx
* Input          : baudrate | parity
* Output         : void
* Return         : None
*******************************************************************************/
void RS485_Configuration(uint32_t baudrate, uint16_t parity)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	USART_ClockInitTypeDef USART_ClockInitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	USART_DeInit(USART1);

	#ifdef  VECT_TAB_RAM
	/* Set the Vector Table base location at 0x20000000 */
	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
	#else  /* VECT_TAB_FLASH  */
	/* Set the Vector Table base location at 0x08000000 */
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
	#endif

	/* Enable the USART1 Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn; // USART1_IRQn
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* Config DE/nRE */
	/* Configure (PA.11) as Output push-pull */
	GPIO_InitStructure.GPIO_Pin = DO_485;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART1 Tx (PA.9) as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART1 Rx (PA.10) as input floating */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	USART_InitStructure.USART_BaudRate = baudrate;
	USART_InitStructure.USART_WordLength = USART_WordLength_9b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;

	if( (parity != USART_Parity_Even) && (parity != USART_Parity_Odd) )
	{
		parity = USART_Parity_No;
	}
	USART_InitStructure.USART_Parity = parity ;

	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	USART_ClockInitStructure.USART_Clock  = USART_Clock_Disable;
	USART_ClockInitStructure.USART_CPOL = USART_CPOL_Low;
	USART_ClockInitStructure.USART_CPHA = USART_CPHA_2Edge;
	USART_ClockInitStructure.USART_LastBit = USART_LastBit_Disable;


	/* Configure the USART1 */
	USART_Init(USART1, &USART_InitStructure);
	USART_ClockInit(USART1,&USART_ClockInitStructure);

	/* Enable the USART Transmoit interrupt: this interrupt is generated when the
	USART1 transmit data register is empty */
//	USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

	/* Enable the USART Receive interrupt: this interrupt is generated when the
	USART1 receive data register is not empty */
	//USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);

	/* Enable the USART1 */
	USART_Cmd(USART1, ENABLE);

	/* Send RS485 in Rece Mode */
	SendMode(FALSE);
}

/*******************************************************************************
* Function Name  : RS485Cmd
* Description    : Control RS485 Tx and Rx
* Input          : bRxEnable | bTxEnable
* Output         : None
* Return         : None
*******************************************************************************/
void RS485Cmd(bool bRxEnable, bool bTxEnable)
{
  if(bRxEnable)
  {
    while(USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET)
    {
    }

    SendMode(FALSE);
    USART_Cmd(USART1, ENABLE);
  }
  else if(bTxEnable)
  {
    SendMode(TRUE);
    USART_Cmd(USART1, ENABLE);

    /* Force to Generate a TXE interrupt */
    /* USART1 transmit data register is empty */
    USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
  }
  else
  {
    SendMode(FALSE);
    USART_Cmd(USART1, DISABLE);
  }
}
