/*
 * SystemInit.c
 *
 *  Created on: 29 pa 2015
 *      Author: SK
 */

#include "SystemInit.h"

volatile TTim SysTimer;


/*******************************************************************************
* Function Name  : RCC_Config
* Description    : Konfiguracja sygnaw taktujcych (72MHz)
* Input          : void
* Output         : void
* Return         : none
*******************************************************************************/
void RCC_Config(void)
{
	ErrorStatus HSEStartUpStatus;  //zmienna opisujaca rezultat uruchomienia HSE

	RCC_DeInit();	                                         //Reset ustawien RCC
	RCC_HSEConfig(RCC_HSE_ON);                               //Wlaczenie HSE
	HSEStartUpStatus = RCC_WaitForHSEStartUp();		         //Odczekaj az HSE bedzie gotowy
	if(HSEStartUpStatus == SUCCESS)
	{
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
		FLASH_SetLatency(FLASH_Latency_2);                   //ustaw zwloke dla pamieci Flash; zaleznie od taktowania rdzenia
	                                                    	 //0:<24MHz; 1:24~48MHz; 2:>48MHz
	    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     //ustaw HCLK=SYSCLK
	    RCC_PCLK2Config(RCC_HCLK_Div1); 					 //ustaw PCLK2=HCLK
	    RCC_PCLK1Config(RCC_HCLK_Div2);					     //ustaw PCLK1=HCLK/2
	    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); //ustaw PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
	    RCC_PLLCmd(ENABLE);                                  //wlacz PLL
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  //odczekaj na poprawne uruchomienie PLL
	    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           //ustaw PLL jako zrodlo sygnalu zegarowego
	    while(RCC_GetSYSCLKSource() != 0x08);                //odczekaj az PLL bedzie sygnalem zegarowym systemu

		/*Tu nalezy umiescic kod zwiazny z konfiguracja sygnalow zegarowych potrzebnych w programie peryferiow*/
	    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);//wlacz taktowanie portu GPIO A	enkoder
	    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);//wlacz taktowanie portu GPIO B	led
	    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);//wlacz taktowanie USART1			uart
	    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	} else {}
}

/*******************************************************************************
* Function Name  : SysTick_Config_Mod
* Description    : Konfiguracja timera SystemTick
* Input          : Dzielnik | Ilosc tickw do odliczenia
* Output         : Stan
* Return         : none
*******************************************************************************/
unsigned long int SysTick_Config_Mod(unsigned long int SysTick_CLKSource, unsigned long int Ticks)
{
	//inicjalizacja licznika SysTick
	//zastepuje funkcje z bibliotek STM w zwiazku z bledem w funcji SysTick_Config
	unsigned long int Settings;

	assert_param( IS_SYSTICK_CLK_SOURCE(SysTick_CLKSource) );

	if (Ticks > SysTick_LOAD_RELOAD_Msk)  return (1);             //Kontrola, czy wartosc poczatkowa nie przekracza max

	SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        //Ustaw wartosc poczatkowa licznika
	NVIC_SetPriority (SysTick_IRQn, 0);                           //Ustaw priorytet przerwania
	SysTick->VAL  = 0;                                            //Ustaw wartosc aktualna licznika
	Settings=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;  //Ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	if( SysTick_CLKSource == SysTick_CLKSource_HCLK )
	{   //Wybierz flage ustawien zrodla sygnalu zegarowego
		Settings |= SysTick_CLKSource_HCLK;
	}

	else
	{
		Settings &= SysTick_CLKSource_HCLK_Div8;
	}
	SysTick->CTRL = Settings;                                     //Zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)

	return (0);
}

/*******************************************************************************
* Function Name  : GPIO_Config
* Description    : Konfiguracja GPIO
* Input          : void
* Output         : void
* Return         : none
********************************************************************************/
void GPIO_Config(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	//LEDS OUTPUTS
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2;	//live & err led
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//INPUTS
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;	//buttons | ENC_INPUT
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

/*******************************************************************************
* Function Name  : NVIC_Config
* Description    : Konfiguracja kontrolera przerwa (priorytety, oraz rda przerwa)
* Input          : void
* Output         : void
* Return         : none
*******************************************************************************/
void NVIC_Config(void)
{
  //Konfigurowanie kontrolera przerwan NVIC
  NVIC_InitTypeDef NVIC_InitStructure;
  EXTI_InitTypeDef EXTI_InitStructure;

#ifdef  VECT_TAB_RAM
  // Jezeli tablica wektorow w RAM, to ustaw jej adres na 0x20000000
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  // VECT_TAB_FLASH
  // W przeciwnym wypadku ustaw na 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif
//  	//rs232
//  	//Wybranie grupy priorytetw
// 	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
//
// 	//Przerwanie od USART1
// 	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
// 	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
// 	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
// 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
// 	NVIC_Init(&NVIC_InitStructure);
//
// 	//Przerwanie dla GPIOA - enkoder
// 	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);                //Wybor modelu grupowania przerwan

 	NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;           //Wybor konfigurowanego IRQ
 	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;      //Priorytet grupowy
 	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;             //Podpriorytet
 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;                //Wlaczenie obslugi IRQ
 	NVIC_Init(&NVIC_InitStructure);

 	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource15);    //Ustawienie zrodla przerwania

 	//Konfiguracja przerwania EXTI0 na linie 0
 	EXTI_InitStructure.EXTI_Line = EXTI_Line15;                    //Wybor linii
 	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
 	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
 	EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
 	EXTI_Init(&EXTI_InitStructure);								   //PRZEKAZANIE STRUKTYRY KONFIGURACYJNEJ DO FUNKCJI INICJUJCEJ
}

/*******************************************************************************
* Function Name  : __reverse
* Description    :
* Input          : void
* Output         : void
* Return         : none
*******************************************************************************/
void _reverse(char* begin,char* end)
{
	char temp;
	while (end>begin)
	{
		temp=*end;
		*end--=*begin;
		*begin++=temp;
	}
}

/*******************************************************************************
* Function Name  : __itoa
* Description    :
* Input          : void
* Output         : void
* Return         : none
*******************************************************************************/
char* _itoa(uint32_t value, char* buffer, int base)
{
	const char digits[]="0123456789abcdef";

	char* buffer_copy=buffer;
	int32_t sign=0;
	int32_t quot,rem;

	if((base>=2)&&(base<=16))				// is the base valid?
	{
		if(base == 10 && (sign = value) < 0)// negative value and base == 10? store the copy (sign)
			value = -value;					// make it positive
		do
		{
			quot=value/base;				// calculate quotient and remainder
			rem=value%base;
			*buffer++ = digits[rem];		// append the remainder to the string
		} while ((value=quot));				// loop while there is something to convert

		if (sign<0)							// was the value negative?
			*buffer++='-';					// append the sign

		_reverse(buffer_copy,buffer-1);	// reverse the string
	}

	*buffer='\0';
	return buffer_copy;
}






